@@ -40,7 +40,6 @@ OPTS = [
 
 LOOPBACK_DEVNAME = 'lo'
 
-IP_NETNS_PATH = '/var/run/netns'
 SYS_NET_PATH = '/sys/class/net'
 DEFAULT_GW_PATTERN = re.compile(r"via (\S+)")
 METRIC_PATTERN = re.compile(r"metric (\S+)")
@@ -246,10 +245,7 @@ class IPWrapper(SubProcessBase):
 
     @classmethod
     def get_namespaces(cls):
-        if not cfg.CONF.AGENT.use_helper_for_ns_read:
-            return os.listdir(IP_NETNS_PATH)
-
-        output = cls._execute([], 'netns', ['list'], run_as_root=True)
+        output = cls._execute([], 'netns', ('list',))
         return [l.split()[0] for l in output.splitlines()]
 
 
@@ -931,11 +927,9 @@ class IpNetnsCommand(IpCommandBase):
                              log_fail_as_error=log_fail_as_error, **kwargs)
 
     def exists(self, name):
-        if not cfg.CONF.AGENT.use_helper_for_ns_read:
-            return name in os.listdir(IP_NETNS_PATH)
-
         output = self._parent._execute(
-            ['o'], 'netns', ['list'], run_as_root=True)
+            ['o'], 'netns', ['list'],
+            run_as_root=cfg.CONF.AGENT.use_helper_for_ns_read)
         for line in [l.split()[0] for l in output.splitlines()]:
             if name == line:
                 return True
