@@ -14,9 +14,11 @@
 
 import mock
 from oslo_config import cfg
+from oslo_db import exception as db_exc
 from oslo_utils import uuidutils
 import sqlalchemy as sa
 from sqlalchemy import orm
+import testtools
 
 from neutron.api.rpc.handlers import l3_rpc
 from neutron.api.v2 import attributes
@@ -94,15 +96,6 @@ class L3HATestFramework(testlib_api.SqlTestCase):
             data['admin_state_up'] = admin_state
         return self.plugin._update_router_db(ctx, router_id, data)
 
-    def _bind_router(self, router_id):
-        with self.admin_ctx.session.begin(subtransactions=True):
-            agents_db = self.plugin.get_agents_db(self.admin_ctx)
-            self.plugin.router_scheduler._bind_ha_router_to_agents(
-                self.plugin,
-                self.admin_ctx,
-                router_id,
-                agents_db)
-
 
 class L3HATestCase(L3HATestFramework):
 
@@ -142,7 +135,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_get_ha_router_port_bindings(self):
         router = self._create_router()
-        self._bind_router(router['id'])
         bindings = self.plugin.get_ha_router_port_bindings(
             self.admin_ctx, [router['id']])
         binding_dicts = [{'router_id': binding['router_id'],
@@ -155,7 +147,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_get_l3_bindings_hosting_router_with_ha_states_ha_router(self):
         router = self._create_router()
-        self._bind_router(router['id'])
         self.plugin.update_routers_states(
             self.admin_ctx, {router['id']: 'active'}, self.agent1['host'])
         bindings = self.plugin.get_l3_bindings_hosting_router_with_ha_states(
@@ -165,8 +156,10 @@ class L3HATestCase(L3HATestFramework):
         self.assertIn((self.agent2['id'], 'standby'), agent_ids)
 
     def test_get_l3_bindings_hosting_router_with_ha_states_agent_none(self):
-        router = self._create_router()
-        # Do not bind router to leave agents as None
+        with mock.patch.object(self.plugin, 'schedule_router'):
+            # Do not bind router to leave agents as None
+            router = self._create_router()
+
         res = self.admin_ctx.session.query(
             l3_hamode_db.L3HARouterAgentPortBinding).filter(
             l3_hamode_db.L3HARouterAgentPortBinding.router_id == router['id']
@@ -192,7 +185,6 @@ class L3HATestCase(L3HATestFramework):
     def test_get_l3_bindings_hosting_router_with_ha_states_active_and_dead(
             self):
         router = self._create_router()
-        self._bind_router(router['id'])
         with mock.patch.object(agents_db.Agent, 'is_active',
                                new_callable=mock.PropertyMock,
                                return_value=False):
@@ -204,6 +196,67 @@ class L3HATestCase(L3HATestFramework):
             agent_ids = [(agent[0]['id'], agent[1]) for agent in bindings]
             self.assertIn((self.agent1['id'], 'standby'), agent_ids)
 
+    def test_router_created_in_active_state(self):
+        router = self._create_router()
+        self.assertEqual(constants.ROUTER_STATUS_ACTIVE, router['status'])
+
+    def test_router_update_stay_active(self):
+        router = self._create_router()
+        router['name'] = 'test_update'
+        router_updated = self.plugin._update_router_db(self.admin_ctx,
+                                                       router['id'], router)
+        self.assertEqual(constants.ROUTER_STATUS_ACTIVE,
+                         router_updated['status'])
+
+    def test_allocating_router_hidden_from_sync(self):
+        r1, r2 = self._create_router(), self._create_router()
+        r1['status'] = constants.ROUTER_STATUS_ALLOCATING
+        self.plugin._update_router_db(self.admin_ctx, r1['id'], r1)
+        # store shorter name for readability
+        get_method = self.plugin._get_active_l3_agent_routers_sync_data
+        # r1 should be hidden
+        expected = [self.plugin.get_router(self.admin_ctx, r2['id'])]
+        self.assertEqual(expected, get_method(self.admin_ctx, None, None,
+                                              [r1['id'], r2['id']]))
+        # but once it transitions back, all is well in the world again!
+        r1['status'] = constants.ROUTER_STATUS_ACTIVE
+        self.plugin._update_router_db(self.admin_ctx, r1['id'], r1)
+        expected.append(self.plugin.get_router(self.admin_ctx, r1['id']))
+        # just compare ids since python3 won't let us sort dicts
+        expected = sorted([r['id'] for r in expected])
+        result = sorted([r['id'] for r in get_method(
+              self.admin_ctx, None, None, [r1['id'], r2['id']])])
+        self.assertEqual(expected, result)
+
+    def test_router_ha_update_allocating_then_active(self):
+        router = self._create_router()
+        _orig = self.plugin._delete_ha_interfaces
+
+        def check_state(context, router_id):
+            self.assertEqual(
+                constants.ROUTER_STATUS_ALLOCATING,
+                self.plugin._get_router(context, router_id)['status'])
+            return _orig(context, router_id)
+        with mock.patch.object(self.plugin, '_delete_ha_interfaces',
+                               side_effect=check_state) as ha_mock:
+            router = self._migrate_router(router['id'], ha=False)
+            self.assertTrue(ha_mock.called)
+        self.assertEqual(constants.ROUTER_STATUS_ACTIVE,
+                         router['status'])
+
+    def test_router_created_allocating_state_during_interface_create(self):
+        _orig = self.plugin._create_ha_interfaces
+
+        def check_state(context, router_db, ha_network):
+            self.assertEqual(constants.ROUTER_STATUS_ALLOCATING,
+                             router_db.status)
+            return _orig(context, router_db, ha_network)
+        with mock.patch.object(self.plugin, '_create_ha_interfaces',
+                               side_effect=check_state) as ha_mock:
+            router = self._create_router()
+            self.assertTrue(ha_mock.called)
+        self.assertEqual(constants.ROUTER_STATUS_ACTIVE, router['status'])
+
     def test_ha_router_create(self):
         router = self._create_router()
         self.assertTrue(router['ha'])
@@ -328,7 +381,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_unbind_ha_router(self):
         router = self._create_router()
-        self._bind_router(router['id'])
 
         bound_agents = self.plugin.get_l3_agents_hosting_routers(
             self.admin_ctx, [router['id']])
@@ -363,7 +415,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_l3_agent_routers_query_interface(self):
         router = self._create_router()
-        self._bind_router(router['id'])
         routers = self.plugin.get_ha_sync_data_for_host(self.admin_ctx,
                                                         self.agent1['host'],
                                                         self.agent1)
@@ -394,7 +445,6 @@ class L3HATestCase(L3HATestFramework):
 
     def _deployed_router_change_ha_flag(self, to_ha):
         router1 = self._create_router(ha=not to_ha)
-        self._bind_router(router1['id'])
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
         router = routers[0]
@@ -425,31 +475,6 @@ class L3HATestCase(L3HATestFramework):
         self._create_router()
         self.assertTrue(self.notif_m.called)
 
-    def test_allocating_router_hidden_from_sync(self):
-        self.plugin.supported_extension_aliases = [
-            constants.L3_HA_MODE_EXT_ALIAS]
-        # simulate a router that is being allocated during
-        # the agent's synchronization
-        r1, r2 = self._create_router(), self._create_router()
-        self.plugin._delete_ha_interfaces(self.admin_ctx, r1['id'])
-        # store shorter name for readability
-        get_method = self.plugin._get_active_l3_agent_routers_sync_data
-        # r1 should be hidden
-        self.assertEqual([r2['id']],
-                         [r['id'] for r in get_method(self.admin_ctx,
-                                                      None, self.agent1,
-                                                      [r1['id'], r2['id']])])
-        # but once it transitions back, all is well in the world again!
-        rdb = self.plugin._get_router(self.admin_ctx, r1['id'])
-        self.plugin._create_ha_interfaces(
-            self.admin_ctx, rdb, self.plugin.get_ha_network(
-                self.admin_ctx, rdb.tenant_id))
-        # just compare ids since python3 won't let us sort dicts
-        expected = sorted([r1['id'], r2['id']])
-        result = sorted([r['id'] for r in get_method(
-              self.admin_ctx, None, self.agent1, [r1['id'], r2['id']])])
-        self.assertEqual(expected, result)
-
     def test_update_router_to_ha_notifies_agent(self):
         router = self._create_router(ha=False)
         self.notif_m.reset_mock()
@@ -457,10 +482,8 @@ class L3HATestCase(L3HATestFramework):
         self.assertTrue(self.notif_m.called)
 
     def test_unique_vr_id_between_routers(self):
-        router1 = self._create_router()
-        router2 = self._create_router()
-        self._bind_router(router1['id'])
-        self._bind_router(router2['id'])
+        self._create_router()
+        self._create_router()
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
         self.assertEqual(2, len(routers))
@@ -472,10 +495,8 @@ class L3HATestCase(L3HATestFramework):
 
     @mock.patch('neutron.db.l3_hamode_db.VR_ID_RANGE', new=set(range(1, 2)))
     def test_vr_id_unique_range_per_tenant(self):
-        router1 = self._create_router()
-        router2 = self._create_router(tenant_id=_uuid())
-        self._bind_router(router1['id'])
-        self._bind_router(router2['id'])
+        self._create_router()
+        self._create_router(tenant_id=_uuid())
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
         self.assertEqual(2, len(routers))
@@ -525,10 +546,8 @@ class L3HATestCase(L3HATestFramework):
         self.assertEqual(allocs_before, allocs_after)
 
     def test_one_ha_router_one_not(self):
-        router1 = self._create_router(ha=False)
-        router2 = self._create_router()
-        self._bind_router(router1['id'])
-        self._bind_router(router2['id'])
+        self._create_router(ha=False)
+        self._create_router()
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
 
@@ -584,6 +603,82 @@ class L3HATestCase(L3HATestFramework):
         networks_after = self.core_plugin.get_networks(self.admin_ctx)
         self.assertEqual(networks_before, networks_after)
 
+    def test_create_ha_interfaces_and_ensure_network_net_exists(self):
+        router = self._create_router()
+        router_db = self.plugin._get_router(self.admin_ctx, router['id'])
+        with mock.patch.object(self.plugin, '_create_ha_network') as create:
+            self.plugin._create_ha_interfaces_and_ensure_network(
+                self.admin_ctx, router_db)
+            self.assertFalse(create.called)
+
+    def test_create_ha_interfaces_and_ensure_network_concurrent_create(self):
+        # create a non-ha router so we can manually invoke the create ha
+        # interfaces call down below
+        router = self._create_router(ha=False)
+        router_db = self.plugin._get_router(self.admin_ctx, router['id'])
+        orig_create = self.plugin._create_ha_network
+        created_nets = []
+
+        def _create_ha_network(*args, **kwargs):
+            # create the network and then raise the error to simulate another
+            # worker creating the network before us.
+            created_nets.append(orig_create(*args, **kwargs))
+            raise db_exc.DBDuplicateEntry(columns=['tenant_id'])
+        with mock.patch.object(self.plugin, '_create_ha_network',
+                               new=_create_ha_network):
+            net = self.plugin._create_ha_interfaces_and_ensure_network(
+                self.admin_ctx, router_db)[1]
+        # ensure that it used the concurrently created network
+        self.assertEqual([net], created_nets)
+
+    def _test_ensure_with_patched_int_create(self, _create_ha_interfaces):
+        # create a non-ha router so we can manually invoke the create ha
+        # interfaces call down below
+        router = self._create_router(ha=False)
+        router_db = self.plugin._get_router(self.admin_ctx, router['id'])
+        with mock.patch.object(self.plugin, '_create_ha_interfaces',
+                               new=_create_ha_interfaces):
+            self.plugin._create_ha_interfaces_and_ensure_network(
+                self.admin_ctx, router_db)
+            self.assertTrue(_create_ha_interfaces.called)
+
+    def test_create_ha_interfaces_and_ensure_network_concurrent_delete(self):
+        orig_create = self.plugin._create_ha_interfaces
+
+        def _create_ha_interfaces(ctx, rdb, ha_net):
+            # concurrent delete on the first attempt
+            if not getattr(_create_ha_interfaces, 'called', False):
+                setattr(_create_ha_interfaces, 'called', True)
+                self.core_plugin.delete_network(self.admin_ctx,
+                                                ha_net['network_id'])
+            return orig_create(ctx, rdb, ha_net)
+        self._test_ensure_with_patched_int_create(_create_ha_interfaces)
+
+    def test_create_ha_interfaces_and_ensure_network_concurrent_swap(self):
+        orig_create = self.plugin._create_ha_interfaces
+
+        def _create_ha_interfaces(ctx, rdb, ha_net):
+            # concurrent delete on the first attempt
+            if not getattr(_create_ha_interfaces, 'called', False):
+                setattr(_create_ha_interfaces, 'called', True)
+                self.core_plugin.delete_network(self.admin_ctx,
+                                                ha_net['network_id'])
+                self.plugin._create_ha_network(self.admin_ctx,
+                                               rdb.tenant_id)
+            return orig_create(ctx, rdb, ha_net)
+
+        self._test_ensure_with_patched_int_create(_create_ha_interfaces)
+
+    def test_create_ha_network_tenant_binding_raises_duplicate(self):
+        router = self._create_router()
+        network = self.plugin.get_ha_network(self.admin_ctx,
+                                             router['tenant_id'])
+        self.plugin._create_ha_network_tenant_binding(
+            self.admin_ctx, 't1', network['network_id'])
+        with testtools.ExpectedException(db_exc.DBDuplicateEntry):
+            self.plugin._create_ha_network_tenant_binding(
+                self.admin_ctx, 't1', network['network_id'])
+
     def test_create_ha_interfaces_binding_failure_rolls_back_ports(self):
         router = self._create_router()
         network = self.plugin.get_ha_network(self.admin_ctx,
@@ -617,7 +712,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_get_active_host_for_ha_router(self):
         router = self._create_router()
-        self._bind_router(router['id'])
         self.assertEqual(
             None,
             self.plugin.get_active_host_for_ha_router(
@@ -631,9 +725,7 @@ class L3HATestCase(L3HATestFramework):
 
     def test_update_routers_states(self):
         router1 = self._create_router()
-        self._bind_router(router1['id'])
         router2 = self._create_router()
-        self._bind_router(router2['id'])
 
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
@@ -653,9 +745,7 @@ class L3HATestCase(L3HATestFramework):
 
     def test_set_router_states_handles_concurrently_deleted_router(self):
         router1 = self._create_router()
-        self._bind_router(router1['id'])
         router2 = self._create_router()
-        self._bind_router(router2['id'])
         bindings = self.plugin.get_ha_router_port_bindings(
             self.admin_ctx, [router1['id'], router2['id']])
         self.plugin.delete_router(self.admin_ctx, router1['id'])
@@ -668,7 +758,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_update_routers_states_port_not_found(self):
         router1 = self._create_router()
-        self._bind_router(router1['id'])
         port = {'id': 'foo', 'device_id': router1['id']}
         with mock.patch.object(self.core_plugin, 'get_ports',
                                return_value=[port]):
@@ -813,7 +902,6 @@ class L3HATestCase(L3HATestFramework):
 
     def test_update_port_status_port_bingding_deleted_concurrently(self):
         router1 = self._create_router()
-        self._bind_router(router1['id'])
         states = {router1['id']: 'active'}
         with mock.patch.object(self.plugin, 'get_ha_router_port_bindings'):
             (self.admin_ctx.session.query(
@@ -883,7 +971,6 @@ class L3HAModeDbTestCase(L3HATestFramework):
         interface_info = {'subnet_id': subnet['id']}
 
         router = self._create_router()
-        self._bind_router(router['id'])
         self.plugin.add_router_interface(self.admin_ctx,
                                          router['id'],
                                          interface_info)
@@ -905,7 +992,6 @@ class L3HAModeDbTestCase(L3HATestFramework):
         interface_info = {'subnet_id': subnet['id']}
 
         router = self._create_router()
-        self._bind_router(router['id'])
         self.plugin.add_router_interface(self.admin_ctx,
                                          router['id'],
                                          interface_info)
